% function cfg = analysis(i_sub,cfg)
%
% This function runs the preprocessing and subject-level statistics
% of one subject i_sub.
% A separate function (analysis2) executes group-level analyses
% All settings are made at the corresponding part of the function or in
% config_subjects.m
% You need to set the spm path for the function to work ("edit analysis.m")
%
% A small trick: If you want to automatically set all values from 0 to 1,
% search for O; and replace by l; You can do this with all if you make sure
% not to use l; or O; later in the script. Rather use l ; and O ; (in this
% example I used L and o instead of 1 and 0).

function cfg = analysis(i_sub,cfg)

%% set preprocessing
cfg.do.dicom = 1;
cfg.do.realign = 1;
cfg.do.plot_realign = 0; % NOT NEEDED FOR PREPROCESSING
cfg.do.slicetiming = 1;
cfg.do.coreg = 1;
cfg.do.segment = 1; % unified segmentation (needed for extracting ROIs)
cfg.do.segment_new = 0; % new segment (needed for DARTEL, DON'T USE)
cfg.do.normalize = 0; % (DON'T USE)
cfg.do.smooth = 0; % (DON'T USE)
cfg.do.dartel_template = 0; % execute me only once (DON'T USE)
cfg.do.dartel_normalize = 0; % cannot be executed without the previous step executed once (DON'T USE)

%% set firstlevel
cfg.do.onsets_create = 1; % will create onsets or overwrite existing onsets
cfg.do.firstlevel_overwrite = 1; % if existing analyses should be overwritten

cfg.do.firstlevel_full = 1;
cfg.do.firstlevel_buttonpresses = 1;
cfg.do.firstlevel_attention = 1;
cfg.do.firstlevel_rule = 1;
cfg.do.firstlevel_cue_fir = 1;

cfg.do.firstlevel_motion_localizer = 1;
cfg.do.firstlevel_color_localizer = 1;

%% smooth unnormalized results and run firstlevel on smoothed data

cfg.do.smooth_unnormalized = 1;

cfg.do.firstlevel_full_s = 1; % s stands for smoothed

%% Create / extract ROIs

cfg.do.extract_mt = 1;
cfg.do.extract_v4 = 1;
cfg.do.create_v1 = 1; % use reverse normalization from anatomical
cfg.do.create_m1 = 1; % use reverse normalization from anatomical

%% Create cfg for all subjects

if exist('cfg','var')
   cfg = config_subjects(cfg);
else
    cfg = config_subjects;
end
   
fpath = mfilename('fullpath');
    [fpath fname fext] = fileparts(fpath); %#ok
    cfg.dirs.function_dir = fpath;

clear cfg_item % remove e.g. when spm12 was loaded in advance

if ispc
    spm_dir = 'C:\Arbeit\Programme\spm8'; % if you use windows
else
    spm_dir = '/home/spm8'; % if you use linux
end

addpath(spm_dir);
addpath(genpath(cfg.dirs.function_dir)); % if you have some stats scripts

% base_dir = cfg.dirs.base_dir;
data_dir = cfg.dirs.data_dir;
% function_dir = cfg.dirs.function_dir;

%% Set paths

if ~exist(data_dir,'dir'), mkdir(data_dir), end


%% ==========================================================
%% PREPROCESSING
%% ==========================================================


%% Import DICOMs

if cfg.do.dicom

fprintf('Importing DICOM files...\n')

    fprintf('\nRunning Subject %i\n\n',i_sub)
    input_dir = fullfile(data_dir,'DICOM',cfg.sub(i_sub).id); % Setting folder where DICOMs are located
    disp('Getting DICOM filenames...')
    P = spm_select('FPListRec',input_dir,'[0-9]');           % Selecting DICOMs (numerical format!)
    P = char(P{:});
    disp('Reading DICOM headers...')
    hdr = spm_dicom_headers_mh(P,true);                       % Reading DICOM headers (small memory usage)
    disp('done!')
    disp('Setting paths and DICOM filenames...')
    [hdr,cfg] = set_dicom_filename(hdr,cfg,i_sub);            % Setting DICOM filenames to own standard
    spm_dicom_convert_mh(hdr,'all','flat',cfg.suffix);

clear hdr; % free memory

disp('done')

end


%% Preprocess data (parameters set in cfg)

%% Realignment

if cfg.do.realign

    prefix = 'f';
    include_loc = 1 ; % should a localizer run (or several) be included in preprocessing?
    
    % Initialize the jobmanager
    spm('defaults','fmri')
    initjobman; % see function at end of script (to initialize only once)

    clear matlabbatch
    
    matlabbatch{1}.spm.spatial.realign.estwrite.data = select_files(cfg,i_sub,prefix,include_loc);
    matlabbatch{1}.spm.spatial.realign.estwrite.eoptions.quality = 0.95;
    matlabbatch{1}.spm.spatial.realign.estwrite.eoptions.sep = 2;
    matlabbatch{1}.spm.spatial.realign.estwrite.eoptions.fwhm = 5;
    matlabbatch{1}.spm.spatial.realign.estwrite.eoptions.rtm = 0 ; % no realignment to mean
    matlabbatch{1}.spm.spatial.realign.estwrite.eoptions.interp = 2;
    matlabbatch{1}.spm.spatial.realign.estwrite.eoptions.wrap = [0 0 0];
    matlabbatch{1}.spm.spatial.realign.estwrite.eoptions.weight = {''};
    matlabbatch{1}.spm.spatial.realign.estwrite.roptions.which = [2 1]; % reslice all plus mean
    matlabbatch{1}.spm.spatial.realign.estwrite.roptions.interp = 4;
    matlabbatch{1}.spm.spatial.realign.estwrite.roptions.wrap = [0 0 0];
    matlabbatch{1}.spm.spatial.realign.estwrite.roptions.mask = 0 ;
    matlabbatch{1}.spm.spatial.realign.estwrite.roptions.prefix = 'r';
    
    spm_jobman('run',matlabbatch)

end


%% Create image of realignment parameters

if cfg.do.plot_realign

    % TODO: Find a way to pass image headers if step is repeated (saves a lot of time)
    % e.g. save headers under parameters (fdecoding-headers, rfdecoding-headers, etc.)
    prefix = 'f';
    plot_on = 1 ;
    plot_rp(cfg,i_sub,prefix,include_loc,plot_on)

end


    
%% Remove bad volumes (replace them by surrounding volumes = bilinear)

% (possibly re-run realignment)


%% Slice timing correction

if cfg.do.slicetiming
    
    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    
    prefix = 'rf';
    
    n_slices = cfg.n_slices;
    TR = cfg.TR;
    
    clear matlabbatch
    
    matlabbatch{1}.spm.temporal.st.scans = select_files(cfg,i_sub,prefix,1);
    matlabbatch{1}.spm.temporal.st.nslices = n_slices;
    matlabbatch{1}.spm.temporal.st.tr = TR;
    matlabbatch{1}.spm.temporal.st.ta = TR - (TR/n_slices);
    switch lower(cfg.sliceorder)
        case 'ascending'
            so = 1:n_slices;
        case 'descending'
            so = n_slices:-1:1 ;
        case 'interleaved'
            if mod(cfg.n_slices,2)
            so = [1:2:n_slices 2:2:n_slices];
            else
                warning('Using settings for Siemens, if no Siemens Scanner is used please find this message and change the script!')
                so = [2:2:n_slices 1:2:n_slices];
            end
        case 'interleaved descending'
            if mod(cfg.n_slices,2)
            so = [n_slices:-2:1 n_slices-1:-2:1];
            else
                warning('Using settings for Siemens, if no Siemens Scanner is used please find this message and change the script!')
                so = [n_slices-1:-2:1 n_slices:-2:1];
            end
    end
    matlabbatch{1}.spm.temporal.st.so = so;
    matlabbatch{1}.spm.temporal.st.refslice = cfg.reference_slice;
    matlabbatch{1}.spm.temporal.st.prefix = 'a';
    spm_jobman('run',matlabbatch)
    
end

%% Do Coregistration

if cfg.do.coreg
    
    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    
    prefix = 'f'; % prefix of mean image
    struct_prefix = 'sstruct';
    
    mean_dir = fullfile(cfg.sub(i_sub).dir,'alldata','run01');
    ref_path = spm_select('fplist',mean_dir,['^mean' prefix cfg.prefix '.*\.img$']);
    if size(ref_path,1)>1, error('More than 1 mean image in %s',mean_dir), end
    ref_path = [ref_path ',1'];
    
    struct_dir = fullfile(cfg.sub(i_sub).dir,'alldata','struct');
    source_path = spm_select('fplist',struct_dir,['^' struct_prefix '.*\.img$']);
    if size(source_path,1)>1, error('More than 1 structural image in %s',struct_dir), end
    source_path = [source_path ',1'];
    
    clear matlabbatch
    
    matlabbatch{1}.spm.spatial.coreg.estimate.ref = {ref_path};
    matlabbatch{1}.spm.spatial.coreg.estimate.source = {source_path};
    matlabbatch{1}.spm.spatial.coreg.estimate.other = {''};
    matlabbatch{1}.spm.spatial.coreg.estimate.eoptions.cost_fun = 'nmi';
    matlabbatch{1}.spm.spatial.coreg.estimate.eoptions.sep = [4 2];
    matlabbatch{1}.spm.spatial.coreg.estimate.eoptions.tol = [0.02 0.02 0.02 0.001 0.001 0.001 0.01 0.01 0.01 0.001 0.001 0.001];
    matlabbatch{1}.spm.spatial.coreg.estimate.eoptions.fwhm = [7 7];
    spm_jobman('run',matlabbatch)
    
end

%% Do Segmentation and write normalized anatomy

if cfg.do.segment

    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    
    struct_prefix = 'sstruct';
    
    struct_dir = fullfile(cfg.sub(i_sub).dir,'alldata','struct');
    struct_path = spm_select('fplist',struct_dir,['^' struct_prefix '.*\.img$']);
    struct_path = [struct_path ',1'];
    
    clear matlabbatch
    
    matlabbatch{1}.spm.spatial.preproc.data = {struct_path};
    matlabbatch{1}.spm.spatial.preproc.output.GM = [0 0 1];
    matlabbatch{1}.spm.spatial.preproc.output.WM = [0 0 1];
    matlabbatch{1}.spm.spatial.preproc.output.CSF = [0 0 0];
    matlabbatch{1}.spm.spatial.preproc.output.biascor = 1 ;
    matlabbatch{1}.spm.spatial.preproc.output.cleanup = 0 ;
    matlabbatch{1}.spm.spatial.preproc.opts.tpm = {
        fullfile(spm_dir,'tpm','grey.nii')
        fullfile(spm_dir,'tpm','white.nii')
        fullfile(spm_dir,'tpm','csf.nii')
        };
    matlabbatch{1}.spm.spatial.preproc.opts.ngaus = [2;2;2;4];
    matlabbatch{1}.spm.spatial.preproc.opts.regtype = 'mni';
    matlabbatch{1}.spm.spatial.preproc.opts.warpreg = 1 ;
    matlabbatch{1}.spm.spatial.preproc.opts.warpco = 25;
    matlabbatch{1}.spm.spatial.preproc.opts.biasreg = 0.0001 ;
    matlabbatch{1}.spm.spatial.preproc.opts.biasfwhm = 60 ;
    matlabbatch{1}.spm.spatial.preproc.opts.samp = 2;
    matlabbatch{1}.spm.spatial.preproc.opts.msk = {''};
    
    spm_jobman('run',matlabbatch)

    clear matlabbatch
    
    % Normalize anatomical (to check with template how well normalization worked)
    normparams_path = spm_select('fplist',struct_dir,[struct_prefix '.*\_seg_sn.mat']);
    matlabbatch{1}.spm.spatial.normalise.write.subj.matname = {normparams_path};
    matlabbatch{1}.spm.spatial.normalise.write.subj.resample = {struct_path};
    matlabbatch{1}.spm.spatial.normalise.write.roptions.preserve = 0 ;
    matlabbatch{1}.spm.spatial.normalise.write.roptions.bb = [-78 -112 -50
                                                               78 76 85];
    matlabbatch{1}.spm.spatial.normalise.write.roptions.vox = [1 1 1];
    matlabbatch{1}.spm.spatial.normalise.write.roptions.interp = 7;
    matlabbatch{1}.spm.spatial.normalise.write.roptions.wrap = [0 1 0];
    matlabbatch{1}.spm.spatial.normalise.write.roptions.prefix = 'w';

    spm_jobman('run',matlabbatch)

end


if cfg.do.segment_new

    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    
    struct_prefix = 'sstruct';
    
    struct_dir = fullfile(cfg.sub(i_sub).dir,'alldata','struct');
    struct_path = spm_select('fplist',struct_dir,['^' struct_prefix '.*\.img$']);
    struct_path = [struct_path ',1'];
    
    clear matlabbatch

    matlabbatch{1}.spm.tools.preproc8.channel.vols = {struct_path};
    matlabbatch{1}.spm.tools.preproc8.channel.biasreg = 0.0001 ;
    matlabbatch{1}.spm.tools.preproc8.channel.biasfwhm = 60 ;
    matlabbatch{1}.spm.tools.preproc8.channel.write = [1 1];
    matlabbatch{1}.spm.tools.preproc8.tissue(1).tpm = {fullfile(spm_dir,'toolbox','Seg','TPM.nii,1')};
    matlabbatch{1}.spm.tools.preproc8.tissue(1).ngaus = 2;
    matlabbatch{1}.spm.tools.preproc8.tissue(1).native = [1 1];
    matlabbatch{1}.spm.tools.preproc8.tissue(1).warped = [0 0];
    matlabbatch{1}.spm.tools.preproc8.tissue(2).tpm = {fullfile(spm_dir,'toolbox','Seg','TPM.nii,2')};
    matlabbatch{1}.spm.tools.preproc8.tissue(2).ngaus = 2;
    matlabbatch{1}.spm.tools.preproc8.tissue(2).native = [1 1];
    matlabbatch{1}.spm.tools.preproc8.tissue(2).warped = [0 0];
    matlabbatch{1}.spm.tools.preproc8.tissue(3).tpm = {fullfile(spm_dir,'toolbox','Seg','TPM.nii,3')};
    matlabbatch{1}.spm.tools.preproc8.tissue(3).ngaus = 2;
    matlabbatch{1}.spm.tools.preproc8.tissue(3).native = [1 1];
    matlabbatch{1}.spm.tools.preproc8.tissue(3).warped = [0 0];
    matlabbatch{1}.spm.tools.preproc8.tissue(4).tpm = {fullfile(spm_dir,'toolbox','Seg','TPM.nii,4')};
    matlabbatch{1}.spm.tools.preproc8.tissue(4).ngaus = 3;
    matlabbatch{1}.spm.tools.preproc8.tissue(4).native = [1 1];
    matlabbatch{1}.spm.tools.preproc8.tissue(4).warped = [0 0];
    matlabbatch{1}.spm.tools.preproc8.tissue(5).tpm = {fullfile(spm_dir,'toolbox','Seg','TPM.nii,5')};
    matlabbatch{1}.spm.tools.preproc8.tissue(5).ngaus = 4;
    matlabbatch{1}.spm.tools.preproc8.tissue(5).native = [1 1];
    matlabbatch{1}.spm.tools.preproc8.tissue(5).warped = [0 0];
    matlabbatch{1}.spm.tools.preproc8.tissue(6).tpm = {fullfile(spm_dir,'toolbox','Seg','TPM.nii,6')};
    matlabbatch{1}.spm.tools.preproc8.tissue(6).ngaus = 2;
    matlabbatch{1}.spm.tools.preproc8.tissue(6).native = [0 0];
    matlabbatch{1}.spm.tools.preproc8.tissue(6).warped = [0 0];
    matlabbatch{1}.spm.tools.preproc8.warp.reg = 4;
    matlabbatch{1}.spm.tools.preproc8.warp.affreg = 'mni';
    matlabbatch{1}.spm.tools.preproc8.warp.samp = 1 ;
    matlabbatch{1}.spm.tools.preproc8.warp.write = [1 1];
    
    spm_jobman('run',matlabbatch)

end

%% For classical firstlevel analysis, normalize and smooth data

if cfg.do.normalize
    
    prefix = 'arf';
    struct_prefix = 'sstruct';
    
    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    clear matlabbatch
    
    struct_dir = fullfile(cfg.sub(i_sub).dir,'alldata','struct');
    normparams_path = spm_select('fplist',struct_dir,[struct_prefix '.*\_seg_sn.mat']);
    matlabbatch{1}.spm.spatial.normalise.write.subj.matname = {normparams_path};
    matlabbatch{1}.spm.spatial.normalise.write.subj.resample = select_files(cfg,i_sub,prefix,1,1);
    matlabbatch{1}.spm.spatial.normalise.write.roptions.preserve = 0 ;
    matlabbatch{1}.spm.spatial.normalise.write.roptions.bb = [-78 -112 -50
                                                              78 76 85];
    matlabbatch{1}.spm.spatial.normalise.write.roptions.vox = [2 2 2];
    matlabbatch{1}.spm.spatial.normalise.write.roptions.interp = 3; % in fact, interpolation doesn't matter so much for normalization
    matlabbatch{1}.spm.spatial.normalise.write.roptions.wrap = [0 0 0];
    matlabbatch{1}.spm.spatial.normalise.write.roptions.prefix = 'w';
    
    spm_jobman('run',matlabbatch)
    
end

if cfg.do.smooth
    
    prefix = 'warf';
    FWHM = cfg.FWHM;
    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    clear matlabbatch
    
    matlabbatch{1}.spm.spatial.smooth.data = select_files(cfg,i_sub,prefix,1,1);
    matlabbatch{1}.spm.spatial.smooth.fwhm = [FWHM FWHM FWHM];
    matlabbatch{1}.spm.spatial.smooth.dtype = 0 ;
    matlabbatch{1}.spm.spatial.smooth.im = 0 ;
    matlabbatch{1}.spm.spatial.smooth.prefix = sprintf('s%02d',FWHM);
    
    spm_jobman('run',matlabbatch)
    
end

%% DARTEL

if cfg.do.dartel_template && i_sub == 1 % make sure to only run once

    if exist(fullfile(cfg.dirs.data_dir, 'sub01','struct','Template0.nii'),'file')
        error('Dartel has already been performed')
    end
    
    ct = 0 ;
    for i_sub = cfg.subject_indices
        ct = ct+1 ;
        rc_dir = fullfile(cfg.sub(i_sub).dir,'alldata','struct');
        rc_path = spm_select('fplist',rc_dir,'^rc1.*\.nii$');
        if isempty(rc_path), error('New segment missing for subject %d !!',i_sub), end
        all_rcs{1}(ct,:) = {[rc_path ',1']};
    end

    spm fmri % because this step takes forever, show display progress
    spm_jobman('initcfg'); % Initialize the jobmanager

    clear matlabbatch

    % WARNING: I adjusted the defaults to be more precise. This step
    % may take 2 days!!!
    
    matlabbatch{1}.spm.tools.dartel.warp.images = all_rcs;
    matlabbatch{1}.spm.tools.dartel.warp.settings.template = 'Template';
    matlabbatch{1}.spm.tools.dartel.warp.settings.rform = 0 ;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(1).its = 4;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(1).rparam = [4 2 1e-06];
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(1).K = 0 ;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(1).slam = 16;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(2).its = 4;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(2).rparam = [2 1 1e-06];
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(2).K = 1 ;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(2).slam = 8;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(3).its = 4;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(3).rparam = [1 0.5 1e-06];
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(3).K = 2;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(3).slam = 4;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(4).its = 4;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(4).rparam = [0.5 0.25 1e-06];
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(4).K = 3;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(4).slam = 2;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(5).its = 4;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(5).rparam = [0.25 0.125 1e-06];
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(5).K = 5;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(5).slam = 1 ;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(6).its = 4;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(6).rparam = [0.25 0.125 1e-06];
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(6).K = 7;
    matlabbatch{1}.spm.tools.dartel.warp.settings.param(6).slam = 0.5;
    matlabbatch{1}.spm.tools.dartel.warp.settings.optim.lmreg = 0.05;
    matlabbatch{1}.spm.tools.dartel.warp.settings.optim.cyc = 8;
    matlabbatch{1}.spm.tools.dartel.warp.settings.optim.its = 4;

    spm_jobman('run',matlabbatch)
    
end

if cfg.do.dartel_normalize
    
    prefix = 'arf';
    struct_prefix = 'sstruct';
    
    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    clear matlabbatch

    % Use voxel dimensions and bounding box of original normalization
    struct_dir = fullfile(cfg.sub(i_sub).dir,'alldata','struct');
    template_name = fullfile(cfg.dirs.sub_dir(1),'struct','Template_6.nii,1'); % lies in folder of subject 1
    flowfield_name = spm_select('FPlist',struct_dir,['^u_rc1' struct_prefix '.*\.nii']);
    flowfield_name = [flowfield_name ',1'];

    matlabbatch{1}.spm.tools.dartel.mni_norm.template = {template_name};
    matlabbatch{1}.spm.tools.dartel.mni_norm.data.subj.flowfield = {flowfield_name};
    matlabbatch{1}.spm.tools.dartel.mni_norm.data.subj.images = select_files(cfg,i_sub,prefix,1,1);
    matlabbatch{1}.spm.tools.dartel.mni_norm.vox = [2 2 2]; % normalized voxel sizes
    matlabbatch{1}.spm.tools.dartel.mni_norm.bb = [-78 -112 -50
                                                    78   76  85];
    matlabbatch{1}.spm.tools.dartel.mni_norm.preserve = 0 ;
    matlabbatch{1}.spm.tools.dartel.mni_norm.fwhm = [6 6 6]; % dartel does smoothing in the same step

    spm_jobman('run',matlabbatch)
    
end

%% ==========================================================
%% FIRSTLEVEL ANALYSES 
%% ==========================================================


%% Firstlevel full (includes most relevant variables)

if cfg.do.firstlevel_full

prefix = 'arf';
onsname = 'full'; % name of onsets file
resname = 'full'; % results path
mparams = 0 ; % do not include movement parameters

standard_firstlevel(cfg,i_sub,prefix,onsname,resname,mparams) % See bottom of the script

% results_dir = fullfile(cfg.sub(i_sub).dir,'results','GLM',resname);
% 
% % Set contrasts
% clear matlabbatch
% matlabbatch{1}.spm.stats.con.spmmat = {fullfile(results_dir,'SPM.mat')};
% matlabbatch{1}.spm.stats.con.consess{1}.tcon.name = 'decision up > decision down';
% matlabbatch{1}.spm.stats.con.consess{1}.tcon.convec = [1 -1];
% matlabbatch{1}.spm.stats.con.consess{1}.tcon.sessrep = 'repl';
% matlabbatch{1}.spm.stats.con.delete = 1 ;
% spm_jobman('run',matlabbatch)

end

%% Firstlevel buttonpresses

if cfg.do.firstlevel_buttonpresses

prefix = 'arf';
onsname = 'buttonpresses'; % name of onsets file
resname = 'buttonpresses'; % results path
mparams = 0 ; % do not include movement parameters

standard_firstlevel(cfg,i_sub,prefix,onsname,resname,mparams) % See bottom of the script

end

%% Firstlevel attention

if cfg.do.firstlevel_attention

prefix = 'arf';
onsname = 'attention'; % name of onsets file
resname = 'attention'; % results path
mparams = 0 ; % do not include movement parameters

standard_firstlevel(cfg,i_sub,prefix,onsname,resname,mparams) % See bottom of the script

end

%% Firstlevel rule

if cfg.do.firstlevel_rule

prefix = 'arf';
onsname = 'rule'; % name of onsets file
resname = 'rule'; % results path
mparams = 0 ; % do not include movement parameters

standard_firstlevel(cfg,i_sub,prefix,onsname,resname,mparams) % See bottom of the script

end

%% Firstlevel rule

if cfg.do.firstlevel_cue_fir

prefix = 'arf';
onsname = 'cue_fir'; % name of onsets file
resname = 'cue_fir'; % results path
mparams = 0 ; % do not include movement parameters
fir = 1; % run fir analysis

standard_firstlevel(cfg,i_sub,prefix,onsname,resname,mparams,fir) % See bottom of the script

end

%% Firstlevel motion localizer (subject space)

if cfg.do.firstlevel_motion_localizer
    
prefix = 'arf';
onsname = 'motion_localizer';
resname = 'motion_localizer';
mparams = 0 ;
n_slices = cfg.n_slices;

% Create onsets if they don't exist yet
if cfg.do.onsets_create
    eval(['fhandle = @getons_' onsname ';'])
    fhandle(cfg,i_sub,1,cfg.do.onsets_create);
    clear fhandle
end

results_dir = fullfile(cfg.sub(i_sub).dir,'results','GLM',resname);
if ~isdir(results_dir), mkdir(results_dir), end
if exist(fullfile(results_dir,'SPM.mat'),'file') && cfg.do.firstlevel_overwrite == 0 
        error('SPM.mat already exists for %s',results_dir)
else
    spm_unlink(fullfile(results_dir,'SPM.mat')); % delete SPM to overwrite without dialog
end

spm('defaults','fmri')
initjobman; % Initialize the jobmanager

% Run model
matlabbatch = firstlevel_localizer(cfg,prefix,results_dir,onsname,i_sub,mparams,n_slices);

spm_jobman('run',matlabbatch)

results_dir = fullfile(cfg.sub(i_sub).dir,'results','GLM',resname);

% Set contrasts
clear matlabbatch
matlabbatch{1}.spm.stats.con.spmmat = {fullfile(results_dir,'SPM.mat')};
matlabbatch{1}.spm.stats.con.consess{1}.tcon.name = 'motion > static';
matlabbatch{1}.spm.stats.con.consess{1}.tcon.convec = [1 -1];
matlabbatch{1}.spm.stats.con.consess{1}.tcon.sessrep = 'repl';
matlabbatch{1}.spm.stats.con.delete = 1 ;

spm_jobman('run',matlabbatch)

end


%% Firstlevel color localizer (subject space)

if cfg.do.firstlevel_color_localizer
    
prefix = 'arf';
onsname = 'color_localizer';
resname = 'color_localizer';
mparams = 0 ;
n_slices = cfg.n_slices;

% Create onsets if they don't exist yet
if cfg.do.onsets_create
    eval(['fhandle = @getons_' onsname ';'])
    fhandle(cfg,i_sub,1,cfg.do.onsets_create);
    clear fhandle
end

results_dir = fullfile(cfg.sub(i_sub).dir,'results','GLM',resname);
if ~isdir(results_dir), mkdir(results_dir), end
if exist(fullfile(results_dir,'SPM.mat'),'file') && cfg.do.firstlevel_overwrite == 0 
        error('SPM.mat already exists for %s',results_dir)
else
    spm_unlink(fullfile(results_dir,'SPM.mat')); % delete SPM to overwrite without dialog
end

spm('defaults','fmri')
initjobman; % Initialize the jobmanager

% Run model
matlabbatch = firstlevel_localizer(cfg,prefix,results_dir,onsname,i_sub,mparams,n_slices);
spm_jobman('run',matlabbatch)

results_dir = fullfile(cfg.sub(i_sub).dir,'results','GLM',resname);

% Set contrasts
clear matlabbatch
matlabbatch{1}.spm.stats.con.spmmat = {fullfile(results_dir,'SPM.mat')};
matlabbatch{1}.spm.stats.con.consess{1}.tcon.name = 'color > gray';
matlabbatch{1}.spm.stats.con.consess{1}.tcon.convec = [1 -1];
matlabbatch{1}.spm.stats.con.consess{1}.tcon.sessrep = 'repl';
matlabbatch{1}.spm.stats.con.delete = 1 ;

spm_jobman('run',matlabbatch)

end

%% Smooth unnormalized data

if cfg.do.smooth_unnormalized
    
    prefix = 'arf';
    FWHM = cfg.FWHM;
    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    clear matlabbatch
    
    matlabbatch{1}.spm.spatial.smooth.data = select_files(cfg,i_sub,prefix,1,1);
    matlabbatch{1}.spm.spatial.smooth.fwhm = [FWHM FWHM FWHM];
    matlabbatch{1}.spm.spatial.smooth.dtype = 0 ;
    matlabbatch{1}.spm.spatial.smooth.im = 0 ;
    matlabbatch{1}.spm.spatial.smooth.prefix = sprintf('s%02d',FWHM);
    
    spm_jobman('run',matlabbatch)
    
end

%% Firstlevel full (includes most relevant variables)

if cfg.do.firstlevel_full_s

prefix = 's06arf';
onsname = 'full'; % name of onsets file
resname = 'full\smoothed'; % results path
mparams = 0 ; % do not include movement parameters

standard_firstlevel(cfg,i_sub,prefix,onsname,resname,mparams) % See bottom of the script

results_dir = fullfile(cfg.sub(i_sub).dir,'results','GLM',resname);

% Set contrasts
clear matlabbatch
matlabbatch{1}.spm.stats.con.spmmat = {fullfile(results_dir,'SPM.mat')};
matlabbatch{1}.spm.stats.con.consess{1}.fcon.name = 'direction vs. color';
matlabbatch{1}.spm.stats.con.consess{1}.fcon.convec = {[1 -1 0 0 0 0 0 0]};
matlabbatch{1}.spm.stats.con.consess{1}.fcon.sessrep = 'repl';
matlabbatch{1}.spm.stats.con.consess{2}.fcon.name = 'up vs. down';
matlabbatch{1}.spm.stats.con.consess{2}.fcon.convec = {[0 0 1 -1 0 0 0 0]};
matlabbatch{1}.spm.stats.con.consess{2}.fcon.sessrep = 'repl';
matlabbatch{1}.spm.stats.con.consess{3}.fcon.name = 'red vs. green';
matlabbatch{1}.spm.stats.con.consess{3}.fcon.convec = {[0 0 0 0 1 -1 0 0]};
matlabbatch{1}.spm.stats.con.consess{3}.fcon.sessrep = 'repl';
matlabbatch{1}.spm.stats.con.consess{4}.fcon.name = 'button left vs. button right';
matlabbatch{1}.spm.stats.con.consess{4}.fcon.convec = {[0 0 0 0 0 0 1 -1]};
matlabbatch{1}.spm.stats.con.consess{4}.fcon.sessrep = 'repl';
matlabbatch{1}.spm.stats.con.delete = 1 ;

spm_jobman('run',matlabbatch)

end 

%% Extract MT (left, right, and both)

if cfg.do.extract_mt
   
    roi_names = {'mt_left','mt_right','mt_both'};
    localizer_name = 'motion_localizer';
        
    n_vox = 200 ; % number of voxels to pick
    sphere_radius = 12; % mm (seems to be ideal)
    
    roi_dir = fullfile(cfg.sub(i_sub).dir,'roi');
    if ~isdir(roi_dir), mkdir(roi_dir), end
    fname = fullfile(cfg.sub(i_sub).dir,'results','GLM',localizer_name,'spmT_0001.img');
    hdr = spm_vol(fname);
    vol = spm_read_vols(hdr);
    
    sz = hdr.dim(1:3);
    [x,y,z] = ndgrid(1:sz(1),1:sz(2),1:sz(3));
    
    sphere_radius = sphere_radius/min(cfg.voxdims); % keep spherical even when voxels are non-isotropic
    proportions = cfg.voxdims/min(cfg.voxdims);
    
    for i = 1:2
        xdist = proportions(1)*(x-cfg.sub(i_sub).roi.(roi_names{i}).position(1)).^2;
        ydist = proportions(2)*(y-cfg.sub(i_sub).roi.(roi_names{i}).position(2)).^2;
        zdist = proportions(3)*(z-cfg.sub(i_sub).roi.(roi_names{i}).position(3)).^2;
        radii = xdist + ydist + zdist;
        % find target values in mask
        tempvol{i}  = radii <= (sphere_radius)^2;
        maskvals = vol(tempvol{i});
        % find cutoff value
        maskvals_sorted = sort(maskvals,'descend');
        cutoff = maskvals_sorted(n_vox);
        % combine both for final mask
        maskvol{i} = vol>=cutoff & tempvol{i};
        
        results_name = fullfile(roi_dir,roi_names{i});
        hdr.dt(1) = 2; % to save space
        hdr.fname = [results_name '.img'];
        spm_write_vol(hdr,maskvol{i});
    end
    
    % combine images for mask "both"
    maskvol_combined = maskvol{1}+maskvol{2};
    results_name = fullfile(roi_dir,roi_names{3});
    hdr.fname = [results_name '.img'];
    spm_write_vol(hdr,maskvol_combined);
        
end



%% Extract V4 (left right, and both)

if cfg.do.extract_v4
    
    roi_names = {'v4_left','v4_right','v4_both'};
    localizer_name = 'color_localizer';
        
    n_vox = 200 ; % number of voxels to pick
    sphere_radius = 12; % mm (seems to be ideal)
    
    roi_dir = fullfile(cfg.sub(i_sub).dir,'roi');
    if ~isdir(roi_dir), mkdir(roi_dir), end
    fname = fullfile(cfg.sub(i_sub).dir,'results','GLM',localizer_name,'spmT_0001.img');
    hdr = spm_vol(fname);
    vol = spm_read_vols(hdr);
    
    sz = hdr.dim(1:3);
    [x,y,z] = ndgrid(1:sz(1),1:sz(2),1:sz(3));
    
    sphere_radius = sphere_radius/min(cfg.voxdims); % keep spherical even when voxels are non-isotropic
    proportions = cfg.voxdims/min(cfg.voxdims);
    
    for i = 1:2
        xdist = proportions(1)*(x-cfg.sub(i_sub).roi.(roi_names{i}).position(1)).^2;
        ydist = proportions(2)*(y-cfg.sub(i_sub).roi.(roi_names{i}).position(2)).^2;
        zdist = proportions(3)*(z-cfg.sub(i_sub).roi.(roi_names{i}).position(3)).^2;
        radii = xdist + ydist + zdist;
        % find target values in mask
        tempvol{i}  = radii <= (sphere_radius)^2;
        maskvals = vol(tempvol{i});
        % find cutoff value
        maskvals_sorted = sort(maskvals,'descend');
        cutoff = maskvals_sorted(n_vox);
        % combine both for final mask
        maskvol{i} = vol>=cutoff & tempvol{i};
        
        results_name = fullfile(roi_dir,roi_names{i});
        hdr.dt(1) = 2; % to save space
        hdr.fname = [results_name '.img'];
        spm_write_vol(hdr,maskvol{i});
    end
    
    % combine images for mask "both"
    maskvol_combined = maskvol{1}+maskvol{2};
    results_name = fullfile(roi_dir,roi_names{3});
    hdr.fname = [results_name '.img'];
    spm_write_vol(hdr,maskvol_combined);
        
end

%% Create bilateral V1

if cfg.do.create_v1
    
    masknum   = {[43 44]}; % numbers for V1 left and V1 right
    roi_names = {'v1'};
    
    % Select anatomical ROI using AAL
    mfname    = mfilename('fullpath');
    aal_fname = fullfile(fileparts(mfname),'templates','aal_MNI_V4.nii');
    aal_hdr   = spm_vol(aal_fname);
    aal_vol   = spm_read_vols(aal_hdr);
    
    for i_roi = 1:length(roi_names)
    
    % Extract voxels with correct number
    aal_vol_masked = ismember(aal_vol,masknum{i_roi});
    aal_fname_masked = fullfile(cfg.sub(i_sub).dir,'roi',[roi_names{i_roi} '.img']);
    aal_hdr_masked = aal_hdr;
    aal_hdr_masked.fname = aal_fname_masked;
    spm_write_vol(aal_hdr_masked,aal_vol_masked);
    
    % Get original voxel size and bounding box from header
    mean_dir = fullfile(cfg.sub(i_sub).dir,'alldata','run01');
    ref_name = spm_select('fplist',mean_dir,['^arf' cfg.prefix '.*001\.img$']);
    
    % Inverse normalize ROI using inverse normalization parameters
    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    clear matlabbatch
    
    struct_dir = fullfile(cfg.sub(i_sub).dir,'alldata','struct');
    normparams_path = spm_select('fplist',struct_dir,'.*\_seg_sn.mat');
    matlabbatch{1}.spm.util.defs.comp{1}.inv.comp{1}.sn2def.matname = {normparams_path};
    matlabbatch{1}.spm.util.defs.comp{1}.inv.comp{1}.sn2def.vox = [NaN NaN NaN];
    matlabbatch{1}.spm.util.defs.comp{1}.inv.comp{1}.sn2def.bb = [NaN NaN NaN
                                                              NaN NaN NaN];
    matlabbatch{1}.spm.util.defs.comp{1}.inv.space = {[ref_name ',1']};
    matlabbatch{1}.spm.util.defs.ofname = '';
    matlabbatch{1}.spm.util.defs.fnames = {[aal_hdr_masked.fname ',1']};
    matlabbatch{1}.spm.util.defs.savedir.saveusr = {fullfile(cfg.sub(i_sub).dir,'roi')};
    matlabbatch{1}.spm.util.defs.interp = 0;
    
    spm_jobman('run',matlabbatch)
    
    % Delete temporary mask
    try
       [fp,fn,fext] = fileparts(aal_hdr_masked.fname);
       newfname = [fp filesep fn];
       oldfname = [fp filesep 'w' fn];
       delete([newfname '.img'])
       delete([newfname '.hdr'])
       movefile([oldfname '.img'],[newfname '.img'])
       movefile([oldfname '.hdr'],[newfname '.hdr'])
    end
        
    end
    
end


%% Create left M1

if cfg.do.create_m1
   
    masknum   = {1}; % number for M1 left
    roi_names = {'m1_left'};

    % Select anatomical ROI using AAL
    mfname    = mfilename('fullpath');
    aal_fname = fullfile(fileparts(mfname),'templates','aal_MNI_V4.nii');
    aal_hdr   = spm_vol(aal_fname);
    aal_vol   = spm_read_vols(aal_hdr);
    
    for i_roi = 1:length(roi_names)
    
    % Extract voxels with correct number
    aal_vol_masked = ismember(aal_vol,masknum{i_roi});
    aal_fname_masked = fullfile(cfg.sub(i_sub).dir,'roi',[roi_names{i_roi} '.img']);
    aal_hdr_masked = aal_hdr;
    aal_hdr_masked.fname = aal_fname_masked;
    spm_write_vol(aal_hdr_masked,aal_vol_masked);
    
    % Get original voxel size and bounding box from header
    mean_dir = fullfile(cfg.sub(i_sub).dir,'alldata','run01');
    ref_name = spm_select('fplist',mean_dir,['^arf' cfg.prefix '.*001\.img$']);
    
    % Inverse normalize ROI using inverse normalization parameters
    spm('defaults','fmri')
    initjobman; % Initialize the jobmanager
    clear matlabbatch
    
    struct_dir = fullfile(cfg.sub(i_sub).dir,'alldata','struct');
    normparams_path = spm_select('fplist',struct_dir,'.*\_seg_sn.mat');
    matlabbatch{1}.spm.util.defs.comp{1}.inv.comp{1}.sn2def.matname = {normparams_path};
    matlabbatch{1}.spm.util.defs.comp{1}.inv.comp{1}.sn2def.vox = [NaN NaN NaN];
    matlabbatch{1}.spm.util.defs.comp{1}.inv.comp{1}.sn2def.bb = [NaN NaN NaN
                                                              NaN NaN NaN];
    matlabbatch{1}.spm.util.defs.comp{1}.inv.space = {[ref_name ',1']};
    matlabbatch{1}.spm.util.defs.ofname = '';
    matlabbatch{1}.spm.util.defs.fnames = {[aal_hdr_masked.fname ',1']};
    matlabbatch{1}.spm.util.defs.savedir.saveusr = {fullfile(cfg.sub(i_sub).dir,'roi')};
    matlabbatch{1}.spm.util.defs.interp = 0;
    
    spm_jobman('run',matlabbatch)
    
    % Delete temporary mask
    try
       [fp,fn,fext] = fileparts(aal_hdr_masked.fname);
       newfname = [fp filesep fn];
       oldfname = [fp filesep 'w' fn];
       delete([newfname '.img'])
       delete([newfname '.hdr'])
       movefile([oldfname '.img'],[newfname '.img'])
       movefile([oldfname '.hdr'],[newfname '.hdr'])
    end
        
    end
    
end



%% Remove all paths
% rmpath(genpath(cfg.dirs.function_dir));


%% SUBFUNCTIONS
function standard_firstlevel(cfg,i_sub,prefix,onsname,resname,mparams,fir)

n_slices = cfg.n_slices;

% Create onsets if they don't exist yet
if cfg.do.onsets_create
    eval(['fhandle = @getons_' onsname ';'])
    fhandle(cfg,i_sub,1,cfg.do.onsets_create);
    clear fhandle
end

results_dir = fullfile(cfg.sub(i_sub).dir,'results','GLM',resname);
if ~isdir(results_dir), mkdir(results_dir), end
if exist(fullfile(results_dir,'SPM.mat'),'file') && cfg.do.firstlevel_overwrite == 0 
    error('SPM.mat already exists for %s',results_dir)
else
    spm_unlink(fullfile(results_dir,'SPM.mat')); % delete SPM to overwrite without dialog
end

spm('defaults','fmri')
initjobman; % Initialize the jobmanager

% Set up model
if exist('fir','var') && fir == 1
    matlabbatch = firstlevel_fir(cfg,prefix,results_dir,onsname,i_sub,mparams,n_slices); % if you need this function, let me know
else
    matlabbatch = firstlevel(cfg,prefix,results_dir,onsname,i_sub,mparams,n_slices); % if you need this function, let me know
end

spm_jobman('run',matlabbatch)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function initjobman

persistent loaded

if isempty(loaded)
    try
        disp('Initializing job manager...')
        spm_jobman('initcfg');
        disp('done.')
    catch
        disp(lasterr)
        error(['Error loading the SPM jobmanager. Maybe a different ',...
            'version of SPM was running previously. Run ''clear classes'' ',...
            'and try again. Otherwise restart Matlab.'])
    end
    loaded = 1 ;
end
